/* convert.c
 *  Copyright: (When this is determined...it will go here)
 *  CVS Info
 *     $Id: $
 *  Overview:
 *     Data conversion functions
 *  Data Structure and Algorithms:
 *  History:
 *  Notes:
 *  References:
 */

#include "parrot/parrot.h"

/* Pack an integer into a string used the specified format */
STRING *
pack_integer(struct Parrot_Interp *interpreter, INTVAL value, UINTVAL type)
{
    unsigned char bytes[8];
    unsigned int len = 0;

    switch (type) {
        case enum_conv_int_8bit:
            bytes[0] = (value & 0xFF);
            len = 1;
            break;
        case enum_conv_int_16bit_le:
            bytes[0] = (value & 0x00FF);
            bytes[1] = (value & 0xFF00) >> 8;
            len = 2;
            break;
        case enum_conv_int_16bit_be:
            bytes[1] = (value & 0x00FF);
            bytes[0] = (value & 0xFF00) >> 8;
            len = 2;
            break;
        case enum_conv_int_32bit_le:
            bytes[0] = (value & 0x000000FF);
            bytes[1] = (value & 0x0000FF00) >>  8;
            bytes[2] = (value & 0x00FF0000) >> 16;
            bytes[3] = (value & 0xFF000000) >> 24;
            len = 4;
            break;
        case enum_conv_int_32bit_be:
            bytes[3] = (value & 0x000000FF);
            bytes[2] = (value & 0x0000FF00) >>  8;
            bytes[1] = (value & 0x00FF0000) >> 16;
            bytes[0] = (value & 0xFF000000) >> 24;
            len = 4;
            break;
    }
    return string_make(interpreter, bytes, len, NULL, 0, NULL);
}

/* Unpack an integer from a string used the specified format */
INTVAL
unpack_integer(struct Parrot_Interp *interpreter, STRING *str, UINTVAL type)
{
    INTVAL value = 0;
    char *bytes = str->strstart;

    switch (type) {
        case enum_conv_int_8bit:
            if (str->bufused >= 1) {
                value = bytes[0];
            }
            break;
        case enum_conv_int_16bit_le:
            if (str->bufused >= 2) {
                value = bytes[1] << 8 | bytes[0];
            }
            break;
        case enum_conv_int_16bit_be:
            if (str->bufused >= 2) {
                value = bytes[0] << 8 | bytes[1];
            }
            break;
        case enum_conv_int_32bit_le:
            if (str->bufused >= 4) {
                value = 
                    bytes[3] << 24 | bytes[2] << 16 | bytes[1] << 8 | bytes[0];
            }
            break;
        case enum_conv_int_32bit_be:
            if (str->bufused >= 4) {
                value = 
                    bytes[0] << 24 | bytes[1] << 16 | bytes[2] << 8 | bytes[3];
            }
            break;
    }
    return value;
}

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 *
 * vim: expandtab shiftwidth=4:
*/
